# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2016-12-20 14:01
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('member', '0022_auto_20161215_1936'),
    ]

    operations = [
        migrations.AlterField(
            model_name='client',
            name='billing_payment_type',
            field=models.CharField(blank=True, choices=[(' ', '----'), ('3rd', '3rd Party'), ('credit', 'Credit card'), ('eft', 'EFT')], max_length=10, null=True, verbose_name='Payment Type'),
        ),
        migrations.AlterField(
            model_name='client',
            name='route',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='member.Route', verbose_name='route'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.6 on 2017-04-04 19:56
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


def separate_shared_address_instances(apps, schema_editor):
    """
    Before changing ForeignKey to OneToOneField, make sure that members use different address instances.
    """
    Member = apps.get_model('member', 'Member')
    address_ids = set()
    for m in Member.objects.select_related('address').all():
        a = m.address
        if a is None:
            continue
        if a.pk not in address_ids:
            address_ids.add(a.pk)
            continue
        else:  # a.pk in address_ids
            print(
                "Member #{} ({} {}) shares the Address #{}. I'm copying this "
                "Address instance and relinking the member to it...".format(
                    m.pk, m.firstname, m.lastname, a.pk
                )
            )
            # Copy this object in DB
            a.pk = None
            a.save()
            assert a.pk not in address_ids
            address_ids.add(a.pk)
            m.address = a
            m.save(update_fields=['address'])


class Migration(migrations.Migration):

    dependencies = [
        ('member', '0028_change_linked_scheduled_status_relationship'),
    ]

    operations = [
        migrations.RunPython(
            separate_shared_address_instances,
            reverse_code=migrations.RunPython.noop
        ),
        migrations.AlterField(
            model_name='member',
            name='address',
            field=models.OneToOneField(null=True, on_delete=django.db.models.deletion.SET_NULL, to='member.Address', verbose_name='address'),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-01-26 16:56
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


def migrate_emergency_contacts(apps, schema_editor):
    # set NotePriority initial
    Client = apps.get_model('member', 'Client')
    EmergencyContact = apps.get_model('member', 'EmergencyContact')
    for client in Client.objects.all():
        if client.emergency_contact:
            EmergencyContact.objects.create(
                client=client,
                member=client.emergency_contact,
                relationship=client.emergency_contact_relationship,
            )


def reverse_emergency_contacts(apps, schema_editor):
    # set NotePriority initial
    Client = apps.get_model('member', 'Client')
    EmergencyContact = apps.get_model('member', 'EmergencyContact')
    for client in Client.objects.all():
        if client.emergencycontact_set.exists():
            # Use as client emergency contact the first item of
            # relationship
            emergency_contact = client.emergencycontact_set.first()
            # reversing info
            client.emergency_contact = emergency_contact.member
            client.emergency_contact_relationship = emergency_contact.relationship
            client.save()
            # removing info
            EmergencyContact.objects.filter(client=client).delete()


class Migration(migrations.Migration):

    dependencies = [
        ('member', '0025_route_vehicle'),
    ]

    operations = [
        migrations.CreateModel(
            name='EmergencyContact',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('relationship', models.CharField(blank=True, max_length=100, null=True, verbose_name='relationship')),
            ],
            options={
                'verbose_name_plural': 'emergency contacts',
            },
        ),
        migrations.AddField(
            model_name='emergencycontact',
            name='client',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='member.Client',
                                    verbose_name='client'),
        ),
        migrations.AddField(
            model_name='emergencycontact',
            name='member',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='member.Member',
                                    verbose_name='member'),
        ),
        migrations.AlterUniqueTogether(
            name='emergencycontact',
            unique_together=set([('client', 'member')]),
        ),
        migrations.AddField(
            model_name='client',
            name='emergency_contacts',
            field=models.ManyToManyField(related_name='emergency_contacts_of', through='member.EmergencyContact',
                                         to='member.Member', verbose_name='emergency contacts'),
        ),
        migrations.RunPython(migrate_emergency_contacts, reverse_emergency_contacts),
        migrations.RemoveField(
            model_name='client',
            name='emergency_contact',
        ),
        migrations.RemoveField(
            model_name='client',
            name='emergency_contact_relationship',
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10 on 2017-01-16 14:41
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


def set_default_data(apps, schema_editor):
    # set for each product the correct language
    NoteCategory = apps.get_model('note', 'NoteCategory')
    Note = apps.get_model('note', 'Note')

    default_category = NoteCategory.objects.get(name="Comments (no actions needed)")

    for note in Note.objects.all():
        note.category = default_category
        note.save()


class Migration(migrations.Migration):

    dependencies = [
        ('note', '0007_notecategory'),
    ]

    operations = [
        migrations.AddField(
            model_name='note',
            name='category',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='notes', to='note.NoteCategory', verbose_name='Category'),
        ),
        migrations.RunPython(set_default_data),
    ]



# coding: utf-8

import collections

from django.contrib.auth.views import login
from django.http import HttpResponseRedirect
from django.contrib.auth.mixins import LoginRequiredMixin
from django.contrib.auth.mixins import PermissionRequiredMixin
from django.urls import reverse_lazy
from django.db.models import Prefetch
from django.views.generic import TemplateView
from souschef.member.models import Client, Route, Client_option, DAYS_OF_WEEK
from souschef.order.models import Order
from datetime import datetime


class HomeView(LoginRequiredMixin, PermissionRequiredMixin, TemplateView):
    permission_required = 'sous_chef.read'
    template_name = 'pages/home.html'

    def get_context_data(self, **kwargs):
        context = super(HomeView, self).get_context_data(**kwargs)

        today = datetime.today()
        active_clients = Client.active.all().count()
        pending_clients = Client.pending.all().count()
        birthday_clients = Client.birthday_contact.get_birthday_boys_and_girls()
        billable_orders = Order.objects.get_billable_orders(
            today.year, today.month
        ).count()
        billable_orders_year = Order.objects.filter(
            status='D',
            delivery_date__year=datetime.today().year).count()
        route_table = self.calculate_route_table()
        context['active_clients'] = active_clients
        context['pending_clients'] = pending_clients
        context['birthday'] = birthday_clients
        context['billable_orders_month'] = billable_orders
        context['billable_orders_year'] = billable_orders_year
        context['routes'] = route_table
        context['total_scheduled_by_day'] = self.calculate_total_scheduled_by_day(route_table)
        context['total_episodic_by_day'] = self.calculate_total_episodic_by_day(route_table)
        return context

    def calculate_route_table(self):
        routes = Route.objects.prefetch_related(Prefetch(
            'client_set',
            to_attr='selected_clients',
            queryset=Client.objects.filter(
                status__in=(Client.ACTIVE, Client.PAUSED, Client.PENDING)
            ).prefetch_related(Prefetch(
                'client_option_set',
                queryset=Client_option.objects.select_related(
                    'option', 'client'
                ).filter(
                    option__name='meals_schedule'
                ).only(
                    'value', 'option__name', 'client', 'client__route'
                )
            )).only(
                'route',
                'meal_default_week',
                'delivery_type'
            )
        )).order_by('name').only('name')

        route_table = []
        for route in routes:
            episodic_defaults = collections.defaultdict(int)
            schedules = collections.defaultdict(int)
            for client in route.selected_clients:
                # meals_schedule are only for ongoing clients
                meals_schedule = dict(client.meals_schedule)

                # For each day, if there's a schedule, count schedule.
                for day, _ in DAYS_OF_WEEK:
                    if day in meals_schedule:
                        schedules[day] += meals_schedule[day].get(
                            'main_dish') or 0

                if client.delivery_type == 'E':  # Episodic
                    meals_default = dict(client.meals_default)
                    for day, _ in DAYS_OF_WEEK:
                        episodic_defaults[day] += meals_default[day].get('main_dish') or 0

            route_table.append((route.name, episodic_defaults, schedules))
        return route_table

    def calculate_total_scheduled_by_day(self, route_table):
        total_scheduled_by_day = {}
        for day, _ in DAYS_OF_WEEK:
            total_scheduled_by_day[day] = 0
            for route_entry in route_table:
                total_scheduled_by_day[day] += route_entry[2][day]
        return total_scheduled_by_day

    def calculate_total_episodic_by_day(self, route_table):
        total_episodic_by_day = {}
        for day, _ in DAYS_OF_WEEK:
            total_episodic_by_day[day] = 0
            for route_entry in route_table:
                total_episodic_by_day[day] += route_entry[1][day]
        return total_episodic_by_day


def custom_login(request):
    if request.user.is_authenticated:
        # Redirect to home if already logged in.
        return HttpResponseRedirect(reverse_lazy("page:home"))
    else:
        return login(request)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-03-22 16:31
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


def migrate_linked_scheduled_status(apps, schema_editor):
    ClientScheduledStatus = apps.get_model('member', 'ClientScheduledStatus')

    for css in ClientScheduledStatus.objects.filter(linked_scheduled_status__isnull=False):
        css.pair = css.linked_scheduled_status
        # remove linked_scheduled_status relationship
        css.linked_scheduled_status = None
        css.save()


def reverse_linked_scheduled_status(apps, schema_editor):
    ClientScheduledStatus = apps.get_model('member', 'ClientScheduledStatus')

    for css in ClientScheduledStatus.objects.filter(pair__isnull=False):
        # make exactly the opposite of 'migrate_linked_scheduled_status'
        css.linked_scheduled_status = css.pair
        # remove pair relationship
        css.pair = None
        css.save()


class Migration(migrations.Migration):
    """
    Change linked_scheduled_status relationship to OneToOneField, because
    the relationship exactly as a 'pair'
    """

    dependencies = [
        ('member', '0027_auto_20170313_1442'),
    ]

    operations = [
        migrations.AddField(
            model_name='clientscheduledstatus',
            name='pair',
            field=models.OneToOneField(
                blank=True, null=True, on_delete=django.db.models.deletion.CASCADE,
                related_name='my_pair', to='member.ClientScheduledStatus',
                verbose_name='Client Scheduled Status Pair'
            ),
        ),
        migrations.RunPython(migrate_linked_scheduled_status, reverse_linked_scheduled_status),
        migrations.RemoveField(
            model_name='clientscheduledstatus',
            name='linked_scheduled_status',
        ),
    ]



from django.core.management.base import BaseCommand
from souschef.order.models import Order, ORDER_STATUS_ORDERED, ORDER_STATUS_DELIVERED
from datetime import datetime
from django.contrib.admin.models import LogEntry, ADDITION


class Command(BaseCommand):
    help = 'Set status to Delivered for all orders that have the status\
            Ordered for the specified delivery date.'

    def add_arguments(self, parser):
        parser.add_argument(
            'delivery_date',
            help='The date must be in the format YYYY-MM-DD',
        )

    def handle(self, *args, **options):
        delivery_date = datetime.strptime(
            options['delivery_date'], '%Y-%m-%d'
        ).date()

        numorders = Order.objects.filter(
            status=ORDER_STATUS_ORDERED,
            delivery_date=delivery_date
        ).update(
            status=ORDER_STATUS_DELIVERED
        )

        # Log the execution
        LogEntry.objects.log_action(
            user_id=1, content_type_id=1,
            object_id="",
            object_repr="Status set to delivered for orders on" + str(
                delivery_date.strftime('%Y-%m-%d %H:%M')),
            action_flag=ADDITION,
        )
        print("Status set to Delivered for {0} orders whose "
              "delivery date is {1}.".format(numorders, delivery_date))



# coding=utf-8
import factory
import random
from souschef.member.factories import ClientFactory
from souschef.note.models import Note, NotePriority, NoteCategory
from django.contrib.auth.models import User


class UserFactory(factory.DjangoModelFactory):

    class Meta:
        model = User

    username = factory.Sequence(lambda x: "user{}".format(x))


class NoteFactory(factory.DjangoModelFactory):

    class Meta:
        model = Note

    note = factory.Faker('sentence')
    author = factory.SubFactory(UserFactory)
    client = factory.SubFactory(ClientFactory)
    priority = factory.LazyAttribute(
        lambda x: random.choice(NotePriority.objects.all())
    )
    category = factory.LazyAttribute(
        lambda x: random.choice(NoteCategory.objects.all())
    )



# -*- coding: utf-8 -*-
# Generated by Django 1.11.29 on 2021-07-23 14:25
from __future__ import unicode_literals

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('note', '0012_note_is_deleted'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='note',
            options={'ordering': ('-date_modified', 'note'), 'verbose_name_plural': 'Notes'},
        ),
        migrations.RenameField(
            model_name='note',
            old_name='date',
            new_name='date_created',
        ),
        migrations.AddField(
            model_name='note',
            name='date_modified',
            field=models.DateTimeField(default=django.utils.timezone.now, verbose_name='Date Modified'),
        ),
        migrations.RunSQL(
            ["UPDATE note_note SET date_modified = date_created"],
            "",
        ),
    ]



import csv

from django.core.management.base import BaseCommand
from souschef.member.models import Client, Member
from souschef.order.models import Order


class Command(BaseCommand):
    help = 'Data: import clients relationships from given csv file.'

    ROW_MID = 0
    ROW_DATE = 1
    ROW_STATUS = 2
    ROW_MAIN_DISH_QUANTITY = 3
    ROW_SIZE = 4
    ROW_FRUIT_SALAD = 5
    ROW_GREEN_SALAD = 6
    ROW_DIABETIC_DESSERT = 7
    ROW_DESSERT = 8
    ROW_PUDDING = 9

    def add_arguments(self, parser):
        parser.add_argument(
            '--file',
            default=False,
            help='Import mock data instead of actual data',
        )

    def handle(self, *args, **options):
        if options['file']:
            file = 'mock_orders.csv'
        else:
            file = 'clients_orders.csv'

        with open(file) as f:
            reader = csv.reader(f, delimiter=';')
            for row in reader:
                try:
                    member = Member.objects.get(mid=row[self.ROW_MID])
                    client = Client.objects.get(member=member)
                    delivery_date = row[self.ROW_DATE]
                    prices = Order.objects.get_client_prices(client)
                    items = {
                        'main_dish_default_quantity':
                            int(row[self.ROW_MAIN_DISH_QUANTITY]),
                        'dessert_default_quantity':
                            int(row[self.ROW_DESSERT]),
                        'diabetic_default_quantity':
                            int(row[self.ROW_DIABETIC_DESSERT]),
                        'fruit_salad_default_quantity':
                            int(row[self.ROW_FRUIT_SALAD]),
                        'green_salad_default_quantity':
                            int(row[self.ROW_GREEN_SALAD]),
                        'pudding_default_quantity':
                            int(row[self.ROW_PUDDING]),
                        'compote_default_quantity': 0,
                        'size_default': row[self.ROW_SIZE],
                    }

                    order = Order.objects.create_order(
                        delivery_date, client, items, prices
                    )

                    order.status = row[self.ROW_STATUS]
                    order.save()

                except Member.DoesNotExist:
                    self.stdout.write(
                        self.style.WARNING('Non existing member'))
                except Client.DoesNotExist:
                    self.stdout.write(
                        self.style.WARNING('Non existing client'))



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-08-16 12:50
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('member', '0033_auto_20170801_1607'),
    ]

    operations = [
        migrations.AlterField(
            model_name='member',
            name='mid',
            field=models.IntegerField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='member',
            name='rid',
            field=models.IntegerField(blank=True, null=True),
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-03-13 18:42
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('meal', '0006_auto_20160826_0007'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='component',
            options={'verbose_name_plural': 'components (Dishes and recipes)'},
        ),
        migrations.AlterModelOptions(
            name='restricted_item',
            options={'verbose_name_plural': 'restricted items (Restriction categories)'},
        ),
        migrations.AlterField(
            model_name='component',
            name='component_group',
            field=models.CharField(choices=[('main_dish', 'Main Dish'), ('dessert', 'Dessert'), ('diabetic', 'Diabetic Dessert'), ('fruit_salad', 'Fruit Salad'), ('green_salad', 'Green Salad'), ('pudding', 'Pudding'), ('compote', 'Compote'), ('sides', 'Sides')], max_length=100, verbose_name='component group'),
        ),
    ]
